"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPluginsSourceExample = exports.injectPluginsCode = exports.createTransformPluginsFileBabelPlugin = void 0;
const path_1 = __importDefault(require("path"));
const fs = __importStar(require("fs-extra"));
const babel = __importStar(require("@babel/core"));
const babelTypes = __importStar(require("@babel/types"));
function tryRequirePrettier() {
    try {
        return require('prettier');
    }
    catch (e) {
        return null;
    }
}
const sharedBabelOptions = {
    // disable user config
    configFile: false,
    babelrc: false,
    presets: [],
    root: process.env.BABEL_TEST_ROOT, // for testing
};
async function transformFileViaPlugin(filePath, babelPlugin) {
    try {
        const initialCode = await fs.readFile(filePath, { encoding: 'utf-8' });
        const updatedResult = await babel.transformAsync(initialCode, {
            filename: path_1.default.basename(filePath),
            filenameRelative: path_1.default.relative(process.cwd(), filePath),
            plugins: [babelPlugin],
            ...sharedBabelOptions,
        });
        if (!updatedResult) {
            return false;
        }
        let finalCode = updatedResult.code;
        if (finalCode === initialCode) {
            return false;
        }
        const maybePrettier = tryRequirePrettier();
        if (maybePrettier && maybePrettier.format) {
            finalCode = maybePrettier.format(finalCode, { parser: 'babel' });
        }
        await fs.writeFile(filePath, finalCode);
        return true;
    }
    catch (e) {
        return false;
    }
}
const returnConfigAst = babel.template.ast('return config; // IMPORTANT to return a config', { preserveComments: true });
function createTransformPluginsFileBabelPlugin(ast) {
    return {
        visitor: {
            Program: (path) => {
                path.unshiftContainer('body', ast.RequireAst);
            },
            Function: (path) => {
                if (!babelTypes.isAssignmentExpression(path.parent)) {
                    return;
                }
                const assignment = path.parent.left;
                const isModuleExports = babelTypes.isMemberExpression(assignment)
                    && babelTypes.isIdentifier(assignment.object)
                    && assignment.object.name === 'module'
                    && babelTypes.isIdentifier(assignment.property)
                    && assignment.property.name === 'exports';
                if (isModuleExports && babelTypes.isFunction(path.parent.right)) {
                    const paramsLength = path.parent.right.params.length;
                    if (paramsLength === 0) {
                        path.parent.right.params.push(babelTypes.identifier('on'));
                        path.parent.right.params.push(babelTypes.identifier('config'));
                    }
                    if (paramsLength === 1) {
                        path.parent.right.params.push(babelTypes.identifier('config'));
                    }
                    const statementToInject = Array.isArray(ast.IfComponentTestingPluginsAst)
                        ? ast.IfComponentTestingPluginsAst
                        : [ast.IfComponentTestingPluginsAst];
                    const ifComponentMode = babelTypes.ifStatement(babelTypes.binaryExpression('===', babelTypes.identifier('config.testingType'), babelTypes.stringLiteral('component')), babelTypes.blockStatement(statementToInject));
                    path.get('body').pushContainer('body', ifComponentMode);
                    if (ast.requiresReturnConfig) {
                        path.get('body').pushContainer('body', returnConfigAst);
                    }
                }
            },
        },
    };
}
exports.createTransformPluginsFileBabelPlugin = createTransformPluginsFileBabelPlugin;
async function injectPluginsCode(pluginsFilePath, ast) {
    return transformFileViaPlugin(pluginsFilePath, createTransformPluginsFileBabelPlugin(ast));
}
exports.injectPluginsCode = injectPluginsCode;
async function getPluginsSourceExample(ast) {
    const exampleCode = [
        'module.exports = (on, config) => {',
        '',
        '}',
    ].join('\n');
    try {
        const babelResult = await babel.transformAsync(exampleCode, {
            filename: 'nothing.js',
            plugins: [createTransformPluginsFileBabelPlugin(ast)],
            ...sharedBabelOptions,
        });
        if (!(babelResult === null || babelResult === void 0 ? void 0 : babelResult.code)) {
            throw new Error();
        }
        return babelResult.code;
    }
    catch (e) {
        throw new Error('Can not generate code example for plugins file because of unhandled error. Please update the plugins file manually.');
    }
}
exports.getPluginsSourceExample = getPluginsSourceExample;
