"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.installFrameworkAdapter = void 0;
const chalk_1 = __importDefault(require("chalk"));
const inquirer_1 = __importDefault(require("inquirer"));
const findPackageJson_1 = require("../findPackageJson");
const utils_1 = require("../utils");
async function guessOrAskForFramework(cwd) {
    // please sort this alphabetically
    const frameworks = {
        react: () => findPackageJson_1.scanFSForAvailableDependency(cwd, { react: '*', 'react-dom': '*' }),
        'vue@2': () => findPackageJson_1.scanFSForAvailableDependency(cwd, { vue: '2.x' }),
        'vue@3': () => findPackageJson_1.scanFSForAvailableDependency(cwd, { vue: '3.x' }),
    };
    const guesses = Object.keys(frameworks).filter((framework) => {
        return frameworks[framework]();
    });
    // found 1 precise guess. Continue
    if (guesses.length === 1) {
        const framework = guesses[0];
        console.log(`\nThis project is using ${chalk_1.default.bold.cyan(framework)}. Let's install the right adapter:`);
        return framework;
    }
    if (guesses.length === 0) {
        console.log(`We were unable to automatically determine your framework 😿. ${chalk_1.default.grey('Make sure to run this command from the directory where your components located in order to make smart detection works. Or continue with manual setup:')}`);
    }
    if (guesses.length > 0) {
        console.log(`It looks like all these frameworks: ${chalk_1.default.yellow(guesses.join(', '))} are available from this directory. ${chalk_1.default.grey('Make sure to run this command from the directory where your components located in order to make smart detection works. Or continue with manual setup:')}`);
    }
    const { framework } = await inquirer_1.default.prompt([
        {
            type: 'list',
            name: 'framework',
            choices: Object.keys(frameworks),
            message: `Which framework do you use?`,
        },
    ]);
    return framework;
}
async function installFrameworkAdapter(cwd, options) {
    const framework = await guessOrAskForFramework(cwd);
    await utils_1.installDependency(`@cypress/${framework}`, options);
    return framework;
}
exports.installFrameworkAdapter = installFrameworkAdapter;
