"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findWebpackConfig = exports.extractWebpackConfigPathFromScript = void 0;
const find_up_1 = __importDefault(require("find-up"));
const path_1 = __importDefault(require("path"));
const findPackageJson_1 = require("../../findPackageJson");
function extractWebpackConfigPathFromScript(script) {
    if (script.includes('webpack ') || script.includes('webpack-dev-server ')) {
        const webpackCliArgs = script.split(' ').map((part) => part.trim());
        const configArgIndex = webpackCliArgs.findIndex((arg) => arg === '--config');
        return configArgIndex === -1 ? null : webpackCliArgs[configArgIndex + 1];
    }
    return null;
}
exports.extractWebpackConfigPathFromScript = extractWebpackConfigPathFromScript;
function findWebpackConfig(root) {
    const webpackConfigPath = find_up_1.default.sync('webpack.config.js', { cwd: root });
    if (webpackConfigPath) {
        return webpackConfigPath;
    }
    const packageJsonIterator = findPackageJson_1.createFindPackageJsonIterator(root);
    const { success, payload } = packageJsonIterator.map(({ scripts }, packageJsonPath) => {
        if (!scripts) {
            return { success: false };
        }
        for (const script of Object.values(scripts)) {
            const webpackConfigRelativePath = extractWebpackConfigPathFromScript(script);
            if (webpackConfigRelativePath) {
                const directoryRoot = path_1.default.resolve(packageJsonPath, '..');
                const webpackConfigPath = path_1.default.resolve(directoryRoot, webpackConfigRelativePath);
                return {
                    success: true,
                    payload: { webpackConfigPath },
                };
            }
        }
        return { success: false };
    });
    return success ? payload === null || payload === void 0 ? void 0 : payload.webpackConfigPath : null;
}
exports.findWebpackConfig = findWebpackConfig;
