"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findInstalledOrInstallCypress = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const find_up_1 = __importDefault(require("find-up"));
const path_1 = __importDefault(require("path"));
const utils_1 = require("./utils");
const chalk_1 = __importDefault(require("chalk"));
const ora_1 = __importDefault(require("ora"));
const initialTemplate = __importStar(require("./initialTemplate"));
async function copyFiles({ ignoreExamples, useTypescript }) {
    let fileSpinner = ora_1.default('Creating config files').start();
    await fs_extra_1.default.outputFile(path_1.default.resolve(process.cwd(), 'cypress.json'), '{}\n');
    await fs_extra_1.default.copy(initialTemplate.getInitialPluginsFilePath(), path_1.default.resolve('cypress', 'plugins/index.js'));
    const supportFiles = await initialTemplate.getInitialSupportFilesPaths();
    await Promise.all(supportFiles.map((supportFilePath) => {
        const newSupportFilePath = path_1.default.resolve('cypress', 'support', path_1.default.basename(supportFilePath));
        return fs_extra_1.default.copy(supportFilePath, newSupportFilePath);
    }));
    if (useTypescript) {
        await fs_extra_1.default.copy(initialTemplate.getInitialTsConfigPath(), path_1.default.resolve('cypress', 'tsconfig.json'));
    }
    // TODO think about better approach
    if (ignoreExamples) {
        const dummySpec = [
            'describe("Spec", () => {',
            '',
            '})',
            '',
        ].join('\n');
        const specFileToCreate = path_1.default.resolve('cypress', 'integration', useTypescript ? 'spec.ts' : 'spec.js');
        await fs_extra_1.default.outputFile(path_1.default.resolve('cypress', 'integration', useTypescript ? 'spec.js' : 'spec.ts'), dummySpec);
        console.log(`In order to ignore examples a spec file ${chalk_1.default.green(path_1.default.relative(process.cwd(), specFileToCreate))}.`);
    }
    fileSpinner.succeed();
}
async function findInstalledOrInstallCypress(options) {
    let cypressJsonPath = await find_up_1.default('cypress.json');
    if (!cypressJsonPath) {
        await utils_1.installDependency('cypress', options);
        await copyFiles(options);
        cypressJsonPath = await find_up_1.default('cypress.json');
    }
    if (!cypressJsonPath) {
        throw new Error('Unexpected error during cypress installation.');
    }
    return {
        cypressConfigPath: cypressJsonPath,
        config: JSON.parse(fs_extra_1.default.readFileSync(cypressJsonPath, { encoding: 'utf-8' }).toString()),
    };
}
exports.findInstalledOrInstallCypress = findInstalledOrInstallCypress;
