"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.installDependency = exports.validateSemverVersion = void 0;
const semver_1 = __importDefault(require("semver"));
const chalk_1 = __importDefault(require("chalk"));
const ora_1 = __importDefault(require("ora"));
const util_1 = __importDefault(require("util"));
const child_process_1 = require("child_process");
/**
 * Compare available version range with the provided version from package.json
 * @param packageName Package name used to display a helper message to user.
 */
function validateSemverVersion(version, allowedVersionRange, packageName) {
    var _a;
    let isValid;
    try {
        const minAvailableVersion = (_a = semver_1.default.minVersion(version)) === null || _a === void 0 ? void 0 : _a.raw;
        isValid = Boolean(minAvailableVersion &&
            semver_1.default.satisfies(minAvailableVersion, allowedVersionRange));
    }
    catch (e) {
        // handle not semver versions like "latest", "git:" or "file:"
        isValid = false;
    }
    if (!isValid && packageName) {
        const packageNameSymbol = chalk_1.default.green(packageName);
        console.warn(`It seems like you are using ${packageNameSymbol} with version ${chalk_1.default.bold(version)}, however we support only ${packageNameSymbol} projects with version ${chalk_1.default.bold(allowedVersionRange)}. \n`);
    }
    return isValid;
}
exports.validateSemverVersion = validateSemverVersion;
async function installDependency(name, options) {
    const commandToRun = options.useYarn ? `yarn add ${name} --dev` : `npm install -D ${name}`;
    let cliSpinner = ora_1.default(`Installing ${name} ${chalk_1.default.gray(`(${commandToRun})`)}`).start();
    try {
        // do this inside function for test stubbing
        const execAsync = util_1.default.promisify(child_process_1.exec);
        await execAsync(commandToRun);
    }
    catch (e) {
        cliSpinner.fail(`Can not install ${name} using ${chalk_1.default.inverse(commandToRun)})}`);
        console.log(e);
        process.exit(1);
    }
    cliSpinner.succeed();
}
exports.installDependency = installDependency;
